function [ ResultadoTest ] = RandomPermutationNSS(DatosUno, DatosDos, NumPermutaciones)
    % Aplicacion del test de permutacion para los datos de entrada, que sern
    % una lista de datos en COLUMNAS cada uno.
    %
    %  Parametros:
    %       DatosUno.- Es la primera serie de datos de dim M
    %       DatosDos.- Sera la segunda serie de datos de dim N
    %

    % Output:
    %       ResultadoTest.- Vector de 4 Elementos; [Mean, Max, Min, p].
    
    % Ejemplo de Datos de entrada (DatosUno y DatosDos)...
    %
    %       23  34  54  26  18  6   ... 34 12
    %       23  26  11  18  ... 4   7
    %
    
    % Ejemplo de Script que permite cargar datos del Workspace ...
    %ExisteDatos = evalin('base','exist(''DatosUno'',''var'')');
    %if(ExisteDatos),    DatosUno = evalin('base','DatosUno'); else,   DatosUno = [5:1:17]';    end
    %ExisteDatosDos = evalin('base','exist(''DatosDos'',''var'')');
    %if(ExisteDatosDos),    DatosDos = evalin('base','DatosDos'); else,   DatosDos = [9:1:18]';    end  
    
    % Datos de ejemplo en caso de ejecutar el script sin parmetros
    if (nargin==0)
        DatosUno = 5:1:17;
        DatosDos = 9:1:21;    
        NumPermutaciones=2000;
    end
    
    ResultadoTest=zeros(1,4);
    ModoDebug=false;

    NdUno=size(DatosUno,2);
    NdDos=size(DatosDos,2);
    NdTotal=NdUno+NdDos;
    DatosTodos=[DatosUno';DatosDos'];

    % Este listado contiene en cada fila una permutacion de los indices
    % No es viable generarlas todas para NdTotal>12, ocuparia cientos de Gigabytes
    % ListaPermutaciones=perms([NdTotal:-1:1]);
    
    % La otra opcion es generar un numero grande de permutaciones aleatorias
    ListaPermutaciones=zeros(NumPermutaciones,NdTotal);
    for Indice=1:NumPermutaciones
        ListaPermutaciones(Indice,:)=randperm(NdTotal);
    end
    
    %DimenTest=size(ListaPermutaciones,1);
    %disp(['Se ha realizado el Test con ' sprintf('%d',DimenTest) ' permutaciones de los datos']);
    %ListaPermutaciones

    % Ahora se Recorren los datos de entrada y se aplican todas las perm.
    VectorTmp=zeros(1,NdTotal);
    ValPermutadoNSS=zeros(1);
    for Indice=1:NumPermutaciones
        % Esto nos devuelve la permutacion j-esima del vector de entrada
        for J=1:NdTotal
           VectorTmp(J)=DatosTodos(ListaPermutaciones(Indice,J));
        end
        VectorTmpUno=VectorTmp(1:NdUno);
        VectorTmpDos=VectorTmp(NdUno+1:NdTotal);

        % Se calculan los histogramas a partir de los valores de saliencia 
        histoUno=zeros(256,1);
        for indTMP=1:length(VectorTmpUno)
            histoUno(VectorTmpUno(indTMP))=histoUno(VectorTmpUno(indTMP))+1;
        end
        histoDos=zeros(256,1);
        for indTMP=1:length(VectorTmpDos)
            histoDos(VectorTmpDos(indTMP))=histoDos(VectorTmpDos(indTMP))+1;
        end

        % Ahora se aplican los calculos de la NSS a esos dos vectores de entrada
        ValPermutadoNSS(Indice)=getNSS(histoUno,histoDos);
    end
    
    % Se hace el histograma y se calcula para que valor la probabilidad es menor que un cierto p-valor
    MinValPerm=min(ValPermutadoNSS);
    MaxValPerm=max(ValPermutadoNSS);
    
    %Inicializacion de parmetros
    ValorNSSSignificativo=0;
    ProbabilidadAlcanzada=0;
    ProbabilidadAlcanzadaMax=0;
    ProbabilidadAlcanzadaMin=0;
    Significacion = 0.01;
    PasoMuestreo=0.001;

    %Se muestra el histograma que se est analizando
    ValoresH=MinValPerm:PasoMuestreo:MaxValPerm;
    HistUnoTMP=histc(ValPermutadoNSS,ValoresH);   
    if(ModoDebug)
        figure(3);set(gcf,'name','Datos histogramas Permutacion NSS');hold off;
        bar(ValoresH,HistUnoTMP);hold on;
        pause(0.1);
    end
    
    %Se va disminuyendo el rango hasta que el area de la cola de las gausianas se hace significativa
    %respecto al area total por ambos lados 
    ValorNSSSignificativoMax=MaxValPerm;
    for Indice=MaxValPerm:-PasoMuestreo:MinValPerm

        HJ=find(ValoresH<Indice);
        HK=find(ValoresH>Indice);
        
        AreaInterior=sum(HistUnoTMP(HJ));         %#ok<FNDSB>
        if(size(HK,2)==0)
            AreaExtremos=0;
        else
            AreaExtremos=sum(HistUnoTMP(HK)); 
        end
        if(AreaInterior==0)
            ProbabilidadMax=1;
        else
            ProbabilidadMax=AreaExtremos/AreaInterior;
        end
        if(ProbabilidadMax>(Significacion/2.0))
            ValorNSSSignificativoMax=Indice;
            ProbabilidadAlcanzadaMax=ProbabilidadMax;
            break;
        end
    end

    ValorNSSSignificativoMin=MinValPerm;
    for Indice=MinValPerm:PasoMuestreo:MaxValPerm

        HJ=find(ValoresH>Indice);
        HK=find(ValoresH<Indice);

        AreaInterior=sum(HistUnoTMP(HJ));         %#ok<FNDSB>
        if(size(HK,2)==0)
            AreaExtremos=0;
        else
            AreaExtremos=sum(HistUnoTMP(HK)); 
        end
        if(AreaInterior==0)
            ProbabilidadMin=1;
        else        
            ProbabilidadMin=AreaExtremos/AreaInterior;
        end
        if(ProbabilidadMin>(Significacion/2))
            ValorNSSSignificativoMin=Indice;
            ProbabilidadAlcanzadaMin=ProbabilidadMin;
            break;
        end
    end    
    
    % Ya que se ha calculado la prob con las dos colas sup e inf, la prob
    % total es la suma de ambas probabilidades
    ProbabilidadAlcanzada=ProbabilidadAlcanzadaMax+ProbabilidadAlcanzadaMin;
    
    %Se devuelven los valores calculados para la significacion del test de Permutacion
    %ResultadoTest=[mean(ValPermutadoNSS);min(ValPermutadoNSS);max(ValPermutadoNSS);ValorNSSSignificativo];
    ResultadoTest=[mean(ValPermutadoNSS);ValorNSSSignificativoMax;ValorNSSSignificativoMin;ProbabilidadAlcanzada];
    
    

% *****************************************************************************************
% En el caso de que supusiesemos una distribucion simetrica se calcularia de este modo
%  
%     for Indice=MaxValPerm:-PasoMuestreo:0
% 
%         HJ=find((ValoresH>(MeanValPerm-Indice))&(ValoresH<(MeanValPerm+Indice)));
%         HK=find((ValoresH<(MeanValPerm-Indice))|(ValoresH>(MeanValPerm+Indice)));
% 
%         AreaInterior=sum(HistUnoTMP(HJ));        
%         if(size(HK,2)==0)
%             AreaExtremos=0;
%         else
%             AreaExtremos=sum(HistUnoTMP(HK)); 
%         end
%         Probabilidad=AreaExtremos/AreaInterior;
%         if(Probabilidad>Significacion)
%             ValorNSSSignificativo=Indice+PasoMuestreo;
%             ProbabilidadAlcanzada=Probabilidad;
%             break;
%         end
%     end    
    
    