% Esta funcion ser la encargada de calcular el valor de la ROC para un cierto intervalo Temporal. Devuelve un valor que
% representa el Indice ROC de ese intervalo. Se puede emplear con los valores de niveles de gris de los mapas de
% saliencia para determinar si la distribucin de TP y FP son semejantes, o con las tasas de disparo de las neuronas,
% para identificar si una neurona diferencia dos caractersticas...
%
%
%   Parametros de Entrada
%
%           ListadoUnoTMP: Es un vector FILA que contiene los niveles de gris del primer conjunto.
%
%           ListadoDosTMP: Es un vector FILA que contiene los niveles de gris del segundo conjunto.
%
%  NOTA: Es posible que para ciertos conjuntos de datos sea necesario adaptar el valor del paso del criterio, por
%  defecto se emplea el valor 0.5.
%
function [IndiceROC] = getPuntoROC(ListadoUnoTMP,ListadoDosTMP)

        ModoDebug=false;   
        ValorNulo=1e-5;        

        % Cdigo para permitir que aunque los listados no contengan datos, EMPTY. No falle el clculo de la ROC.
        if(size(ListadoUnoTMP,2)==0)
            IndiceROC=0;
            return;
        elseif(size(ListadoDosTMP,2)==0)
            IndiceROC=1;
            return;
        end

        % Se obtienen los intervalos para aplicar el criterio
        MaxGlobalTMP=max(max(ListadoUnoTMP),max(ListadoDosTMP));
        MinGlobalTMP=min(min(ListadoUnoTMP),min(ListadoDosTMP));

        NumTRIALsTipoUno=size(ListadoUnoTMP,2);
        NumTRIALsTipoDos=size(ListadoDosTMP,2);

        % Ahora a partir de los histogramas locales se calculan las curvas ROC
        MinCriterio=MinGlobalTMP-1;
        MaxCriterio=MaxGlobalTMP+1;
        AreaFuncCrit=zeros(1,2);
        i=1;
        % El area total de la funcion de distribucion no ser cero en ningn
        % caso ya que siempre habra casos para cero spikes
        AreaTotalUno=NumTRIALsTipoUno;
        AreaTotalDos=NumTRIALsTipoDos;
        PasoCriterio=0.5;
        for ValorCriterio=MinCriterio:PasoCriterio:MaxCriterio
            AreaSupCritUno=length(find(ListadoUnoTMP>=ValorCriterio));
            AreaSupCritDos=length(find(ListadoDosTMP>=ValorCriterio));
            AreaFuncCrit(i,:)=[AreaSupCritUno/AreaTotalUno,AreaSupCritDos/AreaTotalDos];
            i=i+1;
        end

        IndiceROC=trapz(AreaFuncCrit(end:-1:1,1),AreaFuncCrit(end:-1:1,2));

        % Representacin de las dos distribuciones
        if (ModoDebug)
             figure(5);
             subplot(1,4,1:2);hold off;
             ValoresH_TMP=MinCriterio:((MaxCriterio-MinCriterio)/100):MaxCriterio;
             HistUno_TMP=histc(ListadoUnoTMP,ValoresH_TMP);
             HistDos_TMP=histc(ListadoDosTMP,ValoresH_TMP);
             subplot(1,4,1:2),hGrafB=bar(ValoresH_TMP,[HistUno_TMP;HistDos_TMP]',0.9,'grouped');hold on;
             set(hGrafB(1),'FaceColor','r');
             set(hGrafB(2),'FaceColor','b');             
             xlabel('Valor de Saliencia');ylabel('Num casos');     
        end

        % Representacion del area bajo la curva para las dos distribuciones
        if (ModoDebug)
            % Area bajo la curva de las distribuciones Uno y Dos
             subplot(1,4,3);hold off;
             plot(MinCriterio:PasoCriterio:MaxCriterio,AreaFuncCrit(:,1),'r');hold on;
             plot(MinCriterio:PasoCriterio:MaxCriterio,AreaFuncCrit(:,2),'b');hold on;
             axis([MinCriterio,MaxCriterio+ValorNulo,0,1]);
             xlabel('Criterio (Nivel de gris)');ylabel('Areas');     
        end 

        % Se presenta la ROC para esa ventana temporal
        if (ModoDebug)
            % Area Dos frente al Area Uno, Curva ROC de un instante temporal
            subplot(1,4,4);hold off;
            plot(AreaFuncCrit(:,1),AreaFuncCrit(:,2),'g','linewidth',2);hold on;
            axis([0,1,0,1]);
            xlabel('Area Uno');ylabel('Area Dos');
            pause(0.1);
        end
        
        